/*-----------------------------------------------------------------------------

 2002, Jeff M Soft und Hardware GmbH, All Rights Reserved

-----------------------------------------------------------------------------*/

#include "Module.h"
#include "SEMod_struct.h"
#include <assert.h>
#include "SEPin.h"
#include <stdlib.h>

// define some constants to make referencing in/outs clearer

#define PN_SHAPE 0
#define PN_INDEX 1
#define PN_OUT 2

#define NUMSLIDERS 16

void Module::open()
{
	SEModule_base::open();

	// choose which function is used to process audio
	SET_PROCESS_FUNC(Module::sub_process);

	modpos_array = new float [ NUMSLIDERS * sizeof(float) ];

	//getPin(PN_OUT)->TransmitStatusChange( SampleClock(), ST_STATIC);
	getPin(PN_OUT)->TransmitStatusChange( SampleClock(), ST_RUN);
}

Module::Module(seaudioMasterCallback2 seaudioMaster, void *p_resvd1) : SEModule_base(seaudioMaster, p_resvd1)
{
}

// describe the pins (plugs and parameters)
bool Module::getPinProperties (long index, SEPinProperties* properties)
{
	switch( index )								// !!TODO!! list your in / out plugs
	{
	case PN_SHAPE:
		properties->name				= "Array";
		properties->variable_address	= &shape_ptr;
		properties->direction			= DR_IN;
		properties->datatype			= DT_TEXT;
//		properties->default_value		= "";
		properties->flags				= IO_UI_COMMUNICATION_DUAL;
		break;
	case PN_INDEX:
		properties->name				= "Index";
		properties->variable_address	= &input1_buffer;
		properties->direction			= DR_IN;
		properties->datatype			= DT_FSAMPLE;
		break;
	case PN_OUT:
		properties->name				= "Out";
		properties->variable_address	= &output1_buffer;
		properties->direction			= DR_OUT;
		properties->datatype			= DT_FSAMPLE;
		break;
	default:
		return false; // host will ask for plugs 0,1,2,3 etc. return false to signal when done
	};

	return true;
}

void Module::OnPlugStateChange(SEPin *pin)
{
	if( pin->getPinID() == PN_SHAPE )
	{
		parseArrayString( shape_ptr );
		//UpdatePositions(modpos_array, shape_ptr);
		//OnLookupTableChanged();
	}

	//if( pin->getPinID() == PN_INDEX )
	//{
		//ChooseProcess();
		//getPin(PN_OUT)->TransmitStatusChange( SampleClock(), getPin(PN_INDEX)->getStatus() );
	//}
}

// The most important part, processing the audio
void Module::sub_process(long buffer_offset, long sampleFrames )
{
	
	// assign some pointers to your in/output buffers. usually blocks (array) of 96 samples
    float *in1  = buffer_offset + input1_buffer;
    float *out1 = buffer_offset + output1_buffer;

	while(--sampleFrames >= 0) 
    {
		float c = *in1 * 10.f + 0.1f;
		int count = c - 1;
		float x = modpos_array[count];
		*out1 = x;
		in1++;
		out1++;
    }
}

void Module::sub_process_static(long buffer_offset, long sampleFrames )
{
	sub_process(buffer_offset, sampleFrames);
	static_count = static_count - sampleFrames;
	if( static_count <= 0 )
	{
		CallHost(seaudioMasterSleepMode);
	}
}

void Module::UpdatePositions( float *modpos_array, const SeSdkString &p_values)
{	

	SeSdkString s(p_values);

	// convert CString of numbers to array of screen co-ords
	int i = 0;
	while( s.length() > 0 && i < NUMSLIDERS )
	{
		int p1 = s.find("(");
		int p2 = s.find(")");
		if( p2 < 1 )
			return;

		float slider_val = StringToFloat( s.substr(p1+1,p2-1) );// Check this!
		modpos_array[i] = slider_val;

		i++;

		p2++;
		s = s.substr(p2, s.length() - p2);
	}
}

void Module::ChooseProcess()
{
	if( getPin(PN_INDEX)->getStatus() == ST_RUN )
	{
		SET_PROCESS_FUNC(Module::sub_process);
	}
	else
	{
		static_count = getBlockSize();
		SET_PROCESS_FUNC(Module::sub_process_static);
	}
}

float Module::StringToFloat(const SeSdkString &p_string)
{
	char * temp;	// convert string to float
	return (float) strtod( p_string.data(), &temp );
}

Module::~Module()
{
	delete modpos_array;
}

void Module::parseArrayString( char *array_str )	
{
	// for purposes of example, parsed values will be stored in
	//   an array that we will assume has already been allocated

	char *num = array_str; 
	const char *end = array_str + strlen( array_str ); 
	int i = 0;
	while( num < end )
	{
		char *comma = strchr( num, ',' );
		if( comma != NULL )
		{
			*comma = 0; // chop off string at comma position
			modpos_array[i] = StringToFloat( num );
			num = comma + 1; // advance to next number
		}
		/*else // just in case last value has no comma
		{
			num = end; // force end of loop
			modpos_array[i] = StringToFloat( num );
		}*/
		++i;
	}
}