/*-----------------------------------------------------------------------------

 2002, Jeff M Soft und Hardware GmbH, All Rights Reserved

-----------------------------------------------------------------------------*/

#include "Module.h"
#include "SEMod_struct.h"
#include "SEPin.h"
#include <assert.h>

// define some constants to make referencing in/outs clearer
#define PN_TEMPO 0
#define PN_SHUFFLE 1
#define PN_SYNC 2
#define PN_DIVISON 3
#define PN_CLOCK 4
#define PN_RAMP 5

#define SHUFFLE_FACTOR 0.2

// The most important part, processing the audio
void Module::sub_process(long buffer_offset, long sampleFrames )
{
	// assign some pointers to your in/output buffers. usually blocks (array) of 96 samples
	   
	float *in1  = buffer_offset + input1_buffer;
	float *in2  = buffer_offset + input2_buffer;
	float *in3  = buffer_offset + input3_buffer;
    float *out1 = buffer_offset + output1_buffer;
	float *out2 = buffer_offset + output2_buffer;
	float tempo = (*in1 / 60.f) / 2.f;


	switch( divison )
	{
	case 0:
			div_mult = 0.25f;
		break;
	case 1:
			div_mult = 0.5f; 
		break;
	case 2:
			div_mult = 0.6666f;
		break;
	case 3:
			div_mult = 1.f; 
		break;
	case 4:
			div_mult = 1.5f;
		break;
	case 5:
			div_mult = 2.f; 
		break;
	case 6:
			div_mult = 3.f;
		break;
	case 7:
			div_mult = 4.f; 
		break;
	case 8:
			div_mult = 6.f;
		break;
	case 9:
			div_mult = 8.f; 
		break;
	case 10:
			div_mult = 16.f;
		break;
	}


	float rate = div_mult * tempo;
	double shuffle = (*in2 / 10.0) * SHUFFLE_FACTOR;
	int ramp_int = 0;
	double shuffle_left = 0.5 + shuffle;
	double shuffle_right = shuffle_left + 0.03;

	
	if( rate != prev_rate )
	{
		prev_rate = rate;
		delta_ramp = 10.f * rate * sampleRateRecip;
	}

	if( (*in3 != prev_sync) && (*in3 >= 0.4f) )
	{
		ramp = 0.f;
	}

	prev_sync = *in3;

	for( int s = sampleFrames; s > 0; --s )
	{

		ramp_int = (int) ramp;

		ramp = ramp - (double) ramp_int;
		
		//Calculate pulse output
		if( ramp <= 0.03 )
		{
			pulse = 0.5f;
		}
		else if ((ramp >= shuffle_left) && (ramp < shuffle_right))
		{
			pulse = 0.5f;
		}
		else pulse = -0.5f;


		*out1++ = pulse;
		*out2++ = (float) ramp; 

		ramp += delta_ramp;
	
	 }
}

void Module::sub_process_static(long buffer_offset, long sampleFrames )
{
	sub_process(buffer_offset, sampleFrames);
	static_count = static_count - sampleFrames;
	if( static_count <= 0 /* && !AreEventsPending()*/ )
	{
		CallHost(seaudioMasterSleepMode);
	}
}

Module::Module(seaudioMasterCallback2 seaudioMaster, void *p_resvd1) : SEModule_base(seaudioMaster, p_resvd1)
{
}

void Module::open()
{
	// call the base class
	SEModule_base::open();

	// choose which function is used to process audio
	SET_PROCESS_FUNC(Module::sub_process);

	// How to prevent other people using your custom module.
	// query the current registered user's name
	char host_registered_to[100];
	CallHost( seaudioMasterGetRegisteredName, 0,0, host_registered_to );
	bool registered_to_me = strcmp(host_registered_to, "Jeff McClintock") == 0;

	if( !registered_to_me )
	{
		// do something appropriate here if you don't want other people using your module
	}

	sampleRateRecip = 1.0f / sampleRate;
	ramp = 0.0;
	prev_sync = -20.0f;
	prev_rate = -20.0f;
}

Module::~Module()
{
	// This is where you free any memory/resources your module has created
}

// describe your module
bool Module::getModuleProperties (SEModuleProperties* properties)
{
	// describe the plugin, this is the name the end-user will see.
	properties->name = "OL Clock";								// !!TODO!!

	// return a unique string 32 characters max
	// if posible include manufacturer and plugin identity
	// this is used internally by SE to identify the plug.
	// No two plugs may have the same id.
	properties->id = "OL Clock";						// !!TODO!!

	// Info, may include Author, Web page whatever
	properties->about = "by Oli Larkin";								// !!TODO!!
	return true;
}

// describe the pins (plugs and parameters)
bool Module::getPinProperties (long index, SEPinProperties* properties)
{
	switch( index )								// !!TODO!! list your in / out plugs
	{
		// typical input plug (inputs are listed first)
	case 0:
		properties->name				= "Tempo";
		properties->variable_address	= &input1_buffer;
		properties->direction			= DR_IN;
		properties->datatype			= DT_FSAMPLE;
		properties->datatype_extra		= "200,30,200,30";
		properties->default_value		= "120";
		break;
	case 1:
		properties->name				= "Shuffle";
		properties->variable_address	= &input2_buffer;
		properties->direction			= DR_IN;
		properties->datatype			= DT_FSAMPLE;
		properties->datatype_extra		= "100,0,100,0";
		properties->default_value		= "0";
		break;
	case 2:
		properties->name				= "Sync";
		properties->variable_address	= &input3_buffer;
		properties->direction			= DR_IN;
		properties->datatype			= DT_FSAMPLE;
		properties->default_value		= "0";
		break;
	case 3:
		properties->name				= "Division";
		properties->variable_address	= &divison;
		properties->direction			= DR_IN;
		properties->datatype			= DT_ENUM;
		properties->datatype_extra		= "1/1=0,1/2=1,1/3=2,1/4=3,1/6=4,1/8=5,1/12=6,1/16=7,1/24=8,1/32=9,1/64=10";
	//	properties->datatype_extra		= "1/4=1,1/8=2,1/16=4";
		properties->default_value		= "0";
		break;
	case 4:
		properties->name				= "Clock";
		properties->variable_address	= &output1_buffer;
		properties->direction			= DR_OUT;
		properties->datatype			= DT_FSAMPLE;
		break;
	case 5:
		properties->name				= "Gate Ramp";
		properties->variable_address	= &output2_buffer;
		properties->direction			= DR_OUT;
		properties->datatype			= DT_FSAMPLE;
		break;
	default:
		return false; // host will ask for plugs 0,1,2,3 etc. return false to signal when done
	};

	return true;
}


// An input plug has changed value
void Module::OnPlugStateChange(SEPin *pin)
{
	// query the 'state of the input plugs...
	//     ST_RUN     = Normal Streaming Audio        (e.g. from an oscillator)
	//     ST_STATIC  = Fixed, unchanging input value (e.g. a slider at rest)

		state_type out_stat = ST_RUN;

	// 'transmit' new output status to next module 'downstream'
	getPin(PN_CLOCK)->TransmitStatusChange( SampleClock(), out_stat);
	getPin(PN_RAMP)->TransmitStatusChange( SampleClock(), out_stat);
}
