/*-----------------------------------------------------------------------------

 2002, Jeff M Soft und Hardware GmbH, All Rights Reserved

-----------------------------------------------------------------------------*/

#include "Module.h"
#include "SEMod_struct.h"
#include "SEPin.h"
#include <stdlib.h>
#include <assert.h>
#include <math.h>

// define some constants to make referencing in/outs clearer
#define PN_GATERAMP 0
#define PN_SHUFFLE 1
#define PN_GATETIME 2
#define PN_SLIDE 3
#define PN_DOUBLE 4
#define PN_GATE 5
#define PN_SLIDERAMP 6

#define SHUFFLE_FACTOR 0.2f




// The most important part, processing the audio
void Module::sub_process(long buffer_offset, long sampleFrames )
{
	// assign some pointers to your in/output buffers. usually blocks (array) of 96 samples
    float *in1  = buffer_offset + input1_buffer;
	float *in2  = buffer_offset + input2_buffer;
	float *in3  = buffer_offset + input3_buffer;
	float *in4  = buffer_offset + input4_buffer;
    float *in5  = buffer_offset + input5_buffer;
	float *out1 = buffer_offset + output1_buffer;
	float *out2 = buffer_offset + output2_buffer;
	
	float shuffle = (*in2 / 10.f) * SHUFFLE_FACTOR;
	float gatetime = (*in3 / 10.f);
	float shuffle_left = 0.5f + shuffle;
	float shuffle_half = (0.5f - shuffle) / 2.f;
	float half_shuffle_left = shuffle_left * 0.5f;

    while(--sampleFrames >= 0) 
    {
		float ramp = *in1;
			
		// double gates
		if (*in5 > 0.4f)
		{
			if (ramp < 0.125f) gatepulse = 0.5f;
			else if ( (ramp >= 0.25f) && (ramp < 0.375f) ) gatepulse = 0.5f;
			else if ( (ramp >= shuffle_left) && (ramp < shuffle_left + (shuffle_half * 0.5f))) gatepulse = 0.5f;
			else if ( (ramp >= shuffle_left + shuffle_half) && (ramp < shuffle_left + shuffle_half + (shuffle_half * 0.5f))) gatepulse = 0.5f;
			else gatepulse = -0.5f;
		}
		else
		{
			if ( (ramp < (shuffle_left * gatetime)) )
			{
				gatepulse = 0.5f;
			}
			else if ( (ramp >= shuffle_left) && ( ramp < ((0.5f - shuffle) * gatetime) + shuffle_left ) )
			{
				gatepulse = 0.5f;
			}
			else if (*in4 > 0.4f)
			{
				gatepulse = 0.5f;
			}
			else
			{
				gatepulse = -0.5f;
			}
		}

		//Calculate fade_ramp for slide
		if (*in4 > 0.4f)
		{
			if( (ramp >= half_shuffle_left) && (ramp <= shuffle_left))
			{ 
				fade_ramp = (ramp - half_shuffle_left) / (half_shuffle_left);
			}
			else if ( (ramp >= shuffle_left + shuffle_half) && (ramp <= 1.f)  )
			{
				fade_ramp = (ramp - (shuffle_left + shuffle_half)) / shuffle_half;
			}
			else fade_ramp = 0.f;
		}
		else fade_ramp = 0.f;

		*out1 = gatepulse;
		*out2 = fade_ramp;

		in1++;
		in2++;
		in3++;
		in4++;
		in5++;
		out1++;
		out2++;

    }
}

void Module::sub_process_static(long buffer_offset, long sampleFrames )
{
	sub_process(buffer_offset, sampleFrames);
	static_count = static_count - sampleFrames;
	if( static_count <= 0 /* && !AreEventsPending()*/ )
	{
		CallHost(seaudioMasterSleepMode);
	}
}

Module::Module(seaudioMasterCallback2 seaudioMaster, void *p_resvd1) : SEModule_base(seaudioMaster, p_resvd1)
{
}

void Module::open()
{
	// call the base class
	SEModule_base::open();

	// choose which function is used to process audio
	SET_PROCESS_FUNC(Module::sub_process);

	// How to prevent other people using your custom module.
	// query the current registered user's name
	char host_registered_to[100];
	CallHost( seaudioMasterGetRegisteredName, 0,0, host_registered_to );
	bool registered_to_me = strcmp(host_registered_to, "Jeff McClintock") == 0;

	if( !registered_to_me )
	{
		// do something appropriate here if you don't want other people using your module
	}

	gatepulse = -0.5f;
	fade_ramp = 0.f;

}

Module::~Module()
{
}

// describe your module
bool Module::getModuleProperties (SEModuleProperties* properties)
{
	// describe the plugin, this is the name the end-user will see.
	properties->name = "OL Gate Maker";								// !!TODO!!

	// return a unique string 32 characters max
	// if posible include manufacturer and plugin identity
	// this is used internally by SE to identify the plug.
	// No two plugs may have the same id.
	properties->id = "OL Gate Maker";						// !!TODO!!

	// Info, may include Author, Web page whatever
	properties->about = "by Oli Larkin";								// !!TODO!!
	return true;
}

// describe the pins (plugs and parameters)
bool Module::getPinProperties (long index, SEPinProperties* properties)
{
	switch( index )								// !!TODO!! list your in / out plugs
	{
		// typical input plug (inputs are listed first)
	case 0:
		properties->name				= "Gate Ramp";
		properties->variable_address	= &input1_buffer;
		properties->direction			= DR_IN;
		properties->datatype			= DT_FSAMPLE;
		break;
	case 1:
		properties->name				= "Shuffle";
		properties->variable_address	= &input2_buffer;
		properties->direction			= DR_IN;
		properties->datatype			= DT_FSAMPLE;
		break;
	case 2:
		properties->name				= "Gate %";
		properties->variable_address	= &input3_buffer;
		properties->direction			= DR_IN;
		properties->datatype			= DT_FSAMPLE;
		properties->datatype_extra		= "100,0,100,0";
		properties->default_value		= "50";
		break;
	case 3:
		properties->name				= "Slide";
		properties->variable_address	= &input4_buffer;
		properties->direction			= DR_IN;
		properties->datatype			= DT_FSAMPLE;
		break;
	case 4:
		properties->name				= "Double";
		properties->variable_address	= &input5_buffer;
		properties->direction			= DR_IN;
		properties->datatype			= DT_FSAMPLE;
		break;
	case 5:
		properties->name				= "Gate";
		properties->variable_address	= &output1_buffer;
		properties->direction			= DR_OUT;
		properties->datatype			= DT_FSAMPLE;
		break;
	case 6:
		properties->name				= "Slide Ramp";
		properties->variable_address	= &output2_buffer;
		properties->direction			= DR_OUT;
		properties->datatype			= DT_FSAMPLE;
		break;
	default:
		return false; // host will ask for plugs 0,1,2,3 etc. return false to signal when done
	};

	return true;
}

// An input plug has changed value
void Module::OnPlugStateChange(SEPin *pin)
{
	state_type out_stat = ST_RUN;

	//if ( getPin(PN_MODE)->getValue() == 0 ){
    //	    out_stat = ST_STATIC;
    //}
//	else out_stat = ST_RUN;

	getPin(PN_GATE)->TransmitStatusChange( SampleClock(), out_stat);
	getPin(PN_SLIDERAMP)->TransmitStatusChange( SampleClock(), out_stat);

}

